(* -----------------------------------------------------------------------------
   Exemplo de Conferncia
   -----------------------------------------------------------------------------

   Este exemplo mostra como criar um recurso de conferencia (sala)
   e inserir portas nela.

   Na rea de tela "Usurios em Conferncia" o programa permite selecionar
   a sala (recurso) e inserir portas na mesma. Deve-se atentar para o fato
   de existir dois processos para colocar uma porta em conferencia. Um 
   *adicionar* a porta na sala e o outro e *habilitar* a porta na sala.

   Funes relacionadas: (consulte o manual!!)
   ---------------------
    - CreateChatRoom
    - DestroyChatRoom
    - ChatRemovePort
    - ChatAddPort
    - ChatDisablePort
    - ChatEnablePort

----------------------------------------------------------------------------- *)
unit fConferencia;

interface

uses                                                                                                     
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,ShellApi, OleCtrls, voicerlibLib_TLB, ComCtrls,
  ExtCtrls;

type
  TForm1 = class(TForm)
    memStatus: TMemo;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    cboRes: TComboBox;
    Label2: TLabel;
    txtPorta: TEdit;
    cmdInserir: TButton;
    lstPortas: TListBox;
    cmdRemover: TButton;
    cmdHab: TButton;
    cmdDesab: TButton;
    UpDown1: TUpDown;
    lblLista: TLabel;
    lstHabilitadas: TListBox;
    Hab: TLabel;
    GroupBox5: TGroupBox;
    cmdInicializa: TButton;
    cmdFinaliza: TButton;
    txtFirmware: TEdit;
    Label3: TLabel;
    cmdLimpaLog: TButton;
    GroupBox2: TGroupBox;
    cmdCriaConf: TButton;
    cmdDeleteConf: TButton;
    Image1: TImage;
    cmdLeiaMe: TButton;
    voicerlib1: Tvoicerlib;
    procedure cmdInicializaClick(Sender: TObject);
    procedure cmdFinalizaClick(Sender: TObject);
    procedure cmdCriaConfClick(Sender: TObject);
    procedure cmdDeleteConfClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmdInserirClick(Sender: TObject);
    procedure cmdRemoverClick(Sender: TObject);
    procedure cmdHabClick(Sender: TObject);
    procedure cmdDesabClick(Sender: TObject);
    procedure voicerlib1RingDetected(ASender: TObject; Port: Smallint);
    procedure voicerlib1AfterPickup(ASender: TObject; Port: Smallint);
    procedure cboResClick(Sender: TObject);
    procedure voicerlib1DigitDetected(ASender: TObject; Port,
      Digit: Smallint);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cmdLimpaLogClick(Sender: TObject);
    procedure cmdLeiaMeClick(Sender: TObject);
    procedure voicerlib1E1StateChange(ASender: TObject; Port,
      State: Smallint);
    procedure voicerlib1E1Alarm(ASender: TObject; Card, E1Id, AlarmCode,
      AlarmData: Smallint);

  private
    { Private declarations }
  public
    { Public declarations }
    procedure DesligaPorta(Port: integer);
  end;

const
  MAX_CHATROOM = 30;
var
  Form1: TForm1;
  ret: integer;
  hSala: integer;
  ListResource: array[0..MAX_CHATROOM] of TStringList;
  bAtendido: array[1..60] of Boolean;
  nPortaRing : Integer;
  sReturn    : array [0..255] of Char;

implementation

{$R *.dfm}

//copia lista em memoria para lista visual
procedure PreencheLista(Index: integer);
var
  i: integer;
begin
  Form1.lstPortas.Items.Clear;
  for i := 0 To ListResource[Form1.cboRes.ItemIndex].Count -1 do
    Form1.lstPortas.Items.Add(ListResource[Form1.cboRes.ItemIndex].Strings[i]);
end;

procedure MostraStatus(nPorta: integer; sMess: string);
var
  sMsg: string;
begin
  if (nPorta > 0) then
    sMsg := '<' + TimeToStr(Time) + '> - (Porta ' + IntToStr(nPorta) + ') - ' + sMess
  else
    sMsg := '<' + TimeToStr(Time) + '> - ' + sMess;

  Form1.memStatus.Lines.Add(sMsg);
end;

procedure TForm1.cmdInicializaClick(Sender: TObject);
var
  nRet,i,j,k : integer ;

begin
  Screen.Cursor := crHourglass;
  voicerlib1.ConfigPath := txtFirmware.Text;
  //Inicia a Voicerlib
  nRet := voicerlib1.StartVoicerLib;
  //Trata o retorno da inicializacao
  voicerlib1.ReturnCodeToString(nRet,Cardinal(@sReturn));

  if nRet <> DG_EXIT_SUCCESS then
    MostraStatus(0,'Erro de inicializao . cd: ' + IntToStr(ret) + ' (' + sReturn + ') ')
  else
  begin
    MostraStatus(0,'Inicializao da VoicerLib OK. ');
    MostraStatus(0,'Placas Detectadas: ' + IntToStr(voicerlib1.GetCardsCount));
    MostraStatus(0,'Canais disponiveis: ' + IntToStr(voicerlib1.GetPortsCount));
    cmdFinaliza.Enabled := true;
    cmdInicializa.Enabled := false;

    try
      for k := 1 to voicerlib1.GetPortsCount do
      begin
        //Cria thread do protocolo R2D para porta
        nRet := voicerlib1.CreateE1Thread(k);

        case nRet of

          DG_ERROR_DRIVER_CLOSED:
            MostraStatus(k,'O driver no foi iniciado');
          DG_ERROR_THREAD_ALREADY_RUNNING:
            MostraStatus(k,'A thread j havia sido iniciada anteriormente');
          DG_FEATURE_NOT_SUPPORTED:
            MostraStatus(k,'Comando enviado para uma placa FXO e no E1');
          DG_ERROR_FIFO_UNAVAILABLE:
            MostraStatus(k,'Erro na criao da estrutura de comunicao da thread');
          DG_ERROR_COULD_NOT_CREATE_THREAD:
            MostraStatus(k,'Falha na inicializao da thread');

        end;
      end;

      for i := 1 to voicerlib1.GetCardsCount do
      begin
        //Configura o H100 como placa MASTER com sincronismo externo no E1 A (Primeiro E1)
        voicerlib1.SetCardSyncMode(i, SYNC_LINE_A);
        //Configura para que o evento OnE1Alarm ocorra automaticamente sempre quando um alarme for detectado
        voicerlib1.SetAlarmMode(i, ALARM_AUTOMATIC_NOTIFY);
      end;

      for j := 1 to voicerlib1.GetPortsCount do
      begin
        //Categoria do Assinante
        voicerlib1.ConfigE1Thread(j, E1CFG_GROUP_II,1);

        //Seta o Bina
        voicerlib1.SetPortID(j, '1121916363');

        //Numero total de digitos a receber
        voicerlib1.ConfigE1Thread(j, E1CFG_MAXDIGITS_RX,4);

        //Status Grupo B
        voicerlib1.ConfigE1Thread(j, E1CFG_GROUP_B,1);

        //Em ligacoes de entrada, pede bina apos o 2 digito
        voicerlib1.ConfigE1Thread(j, E1CFG_SEND_ID_AFTERDIGIT,2);
      end;

  except
    Application.MessageBox('Erro na Criao das Threads E1', 'Except', mb_OK);
  end;
end;
  Screen.Cursor := crDefault;
end;

procedure TForm1.cmdFinalizaClick(Sender: TObject);
var
  nRet, i : Integer;
begin
  //Termina o tratamento automatico do protocolo R2D de troncos digitais
  for i := 1 to voicerlib1.GetPortsCount do
  begin
    voicerlib1.HangUp(i);//Desliga a porta
    nRet := voicerlib1.DestroyE1Thread(i);

    if nRet <> DG_EXIT_SUCCESS then
      MostraStatus(i,'Erro ao destruir Thread E1');
  end;

  //Finaliza a Voicerlib
  nRet := voicerlib1.ShutdownVoicerLib;
  if nRet <> DG_EXIT_SUCCESS then
    MostraStatus(0,'Erro ao finalizar placa')
  else
    MostraStatus(0,'Placa Finalizada') ;
  cmdInicializa.Enabled := true;
  cmdFinaliza.Enabled := false;
end;

procedure TForm1.cmdCriaConfClick(Sender: TObject);
begin
  {-----------------------------------------------------------------------------
   Cria recurso para conferencia [card, maxport por conferencia]
   so pode usar os canais da placa por isso o parametro de card esta setado em 1
   ----------------------------------------------------------------------------}
  hSala := voicerlib1.CreateChatRoom(1,voicerlib1.GetPortsCount);

  if (hSala > 0) then
  begin
    MostraStatus(0,'Criou sala com handle ' + IntToStr(hSala));
    cboRes.Items.Add(IntToStr(hSala));
    if cboRes.Items.Count = 1 Then
    begin
      cboRes.ItemIndex := 0;
      PreencheLista(cboRes.ItemIndex);
    end;
  end
  else
    MostraStatus(0,'Erro ao criar recurso ' + IntToStr(hSala));
end;

procedure TForm1.cmdDeleteConfClick(Sender: TObject);
var
  j: integer;
  hRes: integer;
begin

  //Pega recurso selecionado
  if cboRes.ItemIndex = - 1 then
    ShowMessage('Selecione um Recurso de Chat na Lista!')
  else
  begin
    hRes := StrToInt(cboRes.Items.Strings[cboRes.ItemIndex]);
    for j := 0 to ListResource[cboRes.ItemIndex].Count -1 do
    begin
      //Remove portas da sala
      voicerlib1.ChatRemovePort(hRes,StrToInt(ListResource[cboRes.ItemIndex].Strings[j]));
      DesligaPorta(j);
    end;
    //Apaga lista
    ListResource[cboRes.ItemIndex].Clear;
    //Exclui recurso
    voicerlib1.DestroyChatRoom(1,hRes);
    cboRes.Items.Delete(cboRes.ItemIndex);
    if cboRes.Items.Count > 0 Then
    begin
      cboRes.ItemIndex := 0;
      PreencheLista(cboRes.ItemIndex);
    end
    else
    begin
      cboRes.ItemIndex := -1;
    end;

    MostraStatus(0,'Excluiu sala com handle ' + IntToStr(hRes));
    
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i,nRet: integer;
  cVersao : array[0..255] of Char;
begin

  for i := 0 to MAX_CHATROOM - 1 do
  begin
    ListResource[i] := TStringList.Create;
  end;

  for i := 1 to 60 do
    bAtendido[i] := false;

  //Verifica a versao da dll e ocx da voicerlib
  nRet := voicerlib1.GetLibVersion(Cardinal(@cVersao),0);
  if nRet = DG_EXIT_SUCCESS then
    MostraStatus(0,'Verso DLL -> ' + cVersao)
  else
    MostraStatus(0,'Verso DLL -> Falha');

  nRet := voicerlib1.GetLibVersion(Cardinal(@cVersao),1);
  if nRet = DG_EXIT_SUCCESS then
    MostraStatus(0,'Verso OCX -> ' + cVersao)
  else
    MostraStatus(0,'Verso OCX -> Falha');

end;

procedure TForm1.cmdInserirClick(Sender: TObject);
var
  hRes: integer;
  i: integer;
begin
  if cboRes.Items.Strings[cboRes.ItemIndex] = '' then
    ShowMessage('Selecione um recurso.');
   
  //Armazena recurso selecionado no combobox
  hRes := StrToInt(cboRes.Items.Strings[cboRes.ItemIndex]);

  //Adiciona no listbox
  lstPortas.Items.Add(txtPorta.Text);
  ListResource[cboRes.ItemIndex].Add(txtPorta.Text);

  //Adiciona porta nos recursos de conferencia
  i := voicerlib1.ChatAddPort(hRes,StrToInt(txtPorta.Text));
  if i = 0 then
    MostraStatus(0,'Adicionou porta ' + txtPorta.Text + ' no recurso ' + IntToStr(hRes))
  else
    MostraStatus(0,'Erro ao adicionar porta no recurso ' + IntToStr(hRes) + ' cdigo ' + IntToStr(i));

  //Mantem porta desabilitada inicialmente
  voicerlib1.ChatDisablePort(hRes,StrToInt(txtPorta.Text));
  txtPorta.Text := inttostr(strtoint(txtPorta.Text) + 1);
end;

procedure TForm1.cmdRemoverClick(Sender: TObject);
var
  hRes,x: integer;
begin
  if lstPortas.Count > 0 then
  begin
    if lstPortas.ItemIndex = -1 then
      lstPortas.ItemIndex := 0;

    hRes := StrToInt(cboRes.Items.Strings[cboRes.ItemIndex]);
    //Remove o canal da conferencia
    x := voicerlib1.ChatRemovePort(hRes,StrToInt(lstPortas.Items.Strings[lstPortas.ItemIndex]));

    if x = 0 Then
    begin
      ListResource[cboRes.ItemIndex].Delete(lstPortas.ItemIndex);
      PreencheLista(cboRes.ItemIndex);
      lstPortas.DeleteSelected;
    end
    else
      MostraStatus(0,'Erro ao tentar remover porta. Cod: ' + inttostr(x) );
  end;
end;

procedure TForm1.cmdHabClick(Sender: TObject);
var
  hRes,x: integer;
begin

  if lstPortas.ItemIndex = -1 Then
    ShowMessage('Seleciona a porta a ser habilitada')
  else
  begin
    hRes := StrToInt(cboRes.Items.Strings[cboRes.ItemIndex]);

    { ------------------------------------------------------------------------
      Ao habilitar uma porta na sala,  possivel passar os seguintes valores
      no parmetro Direction:
        CHAT_TALK = fala na sala mas nao ouve
        CHAT_LISTEN = ouve na sala mas no fala
        CHAT_TALK_LISTEN = ouve e fala no sala
      ------------------------------------------------------------------------}
    x := voicerlib1.ChatEnablePort(hRes,StrToInt(lstPortas.Items.Strings[lstPortas.ItemIndex]),
                                        CHAT_TALK_LISTEN);

    if x <> DG_EXIT_SUCCESS Then
      MostraStatus(0,'No foi possivel habilitar o canal! - cod ' + inttostr(x) )
    else
    begin
      MostraStatus(0,'Porta ' + lstPortas.Items.Strings[lstPortas.ItemIndex] + ' Habilitada');
      lstHabilitadas.Items.Add(lstPortas.Items.Strings[lstPortas.ItemIndex]);
    end;
  end;
end;

procedure TForm1.cmdDesabClick(Sender: TObject);
var
  hRes,x: integer;
begin

  if lstPortas.ItemIndex = -1 Then
    ShowMessage('Seleciona a porta a ser habilitada')
  else
  begin
    hRes := StrToInt(cboRes.Items.Strings[cboRes.ItemIndex]);
    //Desabilita porta da conferencia
    x := voicerlib1.ChatDisablePort(hRes,StrToInt(lstPortas.Items.Strings[lstPortas.ItemIndex]));
    
    if x <> DG_EXIT_SUCCESS Then
      MostraStatus(0,'No foi possivel desabilitar o canal! - cod ' + inttostr(x))
    else
      MostraStatus(0,'Porta ' + lstHabilitadas.Items.Strings[lstHabilitadas.ItemIndex] + ' desabilitada');
  end;
end;

procedure TForm1.voicerlib1RingDetected(ASender: TObject; Port: Smallint);
var
  nRet : Integer;
begin
  MostraStatus(Port,'Ring');
  //atende automaticamente
  nRet := voicerlib1.PickUp(Port,100);
  if nRet <> DG_EXIT_SUCCESS then
    MostraStatus(Port,'Erro ao atender');
  bAtendido[Port] := true;
end;

procedure TForm1.voicerlib1AfterPickup(ASender: TObject; Port: Smallint);
begin
  MostraStatus(Port,'Atendeu');
end;

procedure TForm1.DesligaPorta(Port: integer);
var
  hRes,i,j: integer;
begin
  //Procura porta nas listas
  for i := 0 to MAX_CHATROOM - 1 do
  begin
    for j := 0 to ListResource[i].Count - 1 do
    begin
      if ListResource[i].Strings[j] = IntToStr(Port) then
      begin
        //Pega handle
        hRes := StrToInt(cboRes.Items.Strings[i]);
        //Achou porta
        voicerlib1.ChatDisablePort(hRes,Port);
        //Desliga porta
        voicerlib1.HangUp(Port);
        MostraStatus(Port,'Desabilita e desliga porta ' + IntToStr(Port));
      end;
    end;
  end;
end;

procedure TForm1.cboResClick(Sender: TObject);
begin
  PreencheLista(cboRes.ItemIndex);
end;

procedure TForm1.voicerlib1DigitDetected(ASender: TObject; Port,
  Digit: Smallint);
begin
  MostraStatus(Port,'Digito ' + Chr(Digit));
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  //verifica se o driver esta habilitado
  if voicerlib1.DriverEnabled then
  begin
    Application.MessageBox('Voc deve finalizar o driver antes de fechar a aplicao','Aviso',MB_OK);
    CanClose := False;
  end;
end;

procedure TForm1.cmdLimpaLogClick(Sender: TObject);
begin
  memStatus.Clear;
end;

procedure TForm1.cmdLeiaMeClick(Sender: TObject);
var
  L: String;
begin
  L:= ExtractFilePath(Application.ExeName) + 'Leiame.txt';
  ShellExecute (0 , nil , PChar(L), nil, nil, SW_NORMAL);

end;

procedure TForm1.voicerlib1E1StateChange(ASender: TObject; Port,
  State: Smallint);
begin

  case State of
    C_SEIZURE:
    begin
        MostraStatus(Port,'E1 Ocupao');
        //voicerlib1.SetStartE1RxCount(Port, 4);
    end;

    C_NUMBER_RECEIVED:
        MostraStatus(Port,'E1 Nmero Recebido');

    C_CONGESTION:
        MostraStatus(Port,'E1 Congestionamento');

    C_B_ENDCALL:
        MostraStatus(Port,'E1 Fim da Chamada');

    C_NOTCOMPLETED:
        MostraStatus(Port,'E1 No completada');

    C_GROUP_II:
        MostraStatus(Port,'E1 Categoria');

    C_UNAVAILABLE:
        MostraStatus(Port,'E1 No disponvel');

  end;
end;

procedure TForm1.voicerlib1E1Alarm(ASender: TObject; Card, E1Id, AlarmCode,
  AlarmData: Smallint);
begin
  case AlarmCode of
    ALARM_RSLIP:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Escorregamento (problema de sincronismo) - ' + IntToStr(AlarmCode));
    end;

    ALARM_RAIS:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Alarme remoto ON')
      else
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Alarme remoto OFF');
    end;

    ALARM_AISS:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Indicao de alarme ON')
      else
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Indicao de alarme OFF');
    end;

    ALARM_AIS16S:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Indicao de alarme canal 16 ON')
      else
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Indicao de alarme canal 16 OFF');
    end;

    ALARM_LOSS:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Perda de sinal ON')
      else
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Perda de sinal OFF');
    end;

    ALARM_RESERVED:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Reservado ON')
      else
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Reservado OFF');
    end;

    ALARM_MFSYNC:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Sincronismo de multiquadro ON')
      else
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Sincronismo de multiquadro OFF');
    end;

    ALARM_SYNC:
    begin
      if AlarmData = 1 then
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Sincronismo de quadro ON')
      else
        MostraStatus(0,'Placa ' + IntToStr(Card)+ ' ->Sincronismo de quadro OFF');
    end;
  end;
end;

end.
